using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation;

namespace CameraViewer
{
    public class CameraImage : UIElement
    {

        private ICameraSource _source;
        private CameraEventHandler cameraHandler;
        private Bitmap frame;

        public CameraImage()
        {
            cameraHandler = onCameraFrame;
        }

        private void onCameraFrame(Bitmap bitmap)
        {
            this.frame = bitmap;
            this.Dispatcher.BeginInvoke(new DispatcherOperationCallback(InvalidateRender), null);
        }

        private object InvalidateRender(object arg)
        {
            this.Invalidate();
            return null;
        }

        public override void OnRender(Microsoft.SPOT.Presentation.Media.DrawingContext dc)
        {
            if (this.frame != null)
                dc.DrawImage(this.frame, 0, 0, 0, 0, this.ActualWidth, this.ActualHeight);
        }

        protected override void MeasureOverride(int availableWidth, int availableHeight, out int desiredWidth, out int desiredHeight)
        {
            desiredWidth = availableWidth;
            desiredHeight = availableHeight;
        }

        public ICameraSource Source
        {
            get { return _source; }
            set
            {
                if (_source != null)
                {
                    _source.CameraFrame -= cameraHandler;
                    _source.Stop();
                }
                _source = value;
                if (_source != null)
                {
                    _source.CameraFrame += cameraHandler;
                    _source.Start();
                }
            }
        }

    }
}
