using System;
using Microsoft.SPOT;
using System.Net;
using System.Threading;
using System.IO;
using System.Text;
using Microsoft.SPOT.Hardware;

namespace CameraViewer
{
    public class JpegSource : ICameraSource
    {

        private Uri uri;
        private Thread loadThread;

        public JpegSource(Uri uri)
        {
            this.uri = uri;
        }

        #region ICameraSource Members

        public event CameraEventHandler CameraFrame;

        public void Stop()
        {
            if (loadThread != null)
            {
                loadThread.Abort();
                loadThread.Join();
            }
        }

        public void Start()
        {
            if (loadThread == null)
            {
                loadThread = new Thread(LoadFrames);
                loadThread.Start();
            }
        }

        #endregion

        private void LoadFrames()
        {
            try
            {
                while (true)
                {
                    // Creo la richiesta HTTP in base all'uri
                    WebRequest request = HttpWebRequest.Create(this.uri);
                    // Ottengo la risposta
                    HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                    // Guardo se  arrivato un status code 200
                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        LoadResponse(response);
                    }

                }
            }
            finally
            {
                loadThread = null;
            }
        }

        private void LoadResponse(HttpWebResponse response)
        {
            if (CameraFrame == null) return;

            long l = response.ContentLength;
            if (l > 0)
            {
                Stream stream = response.GetResponseStream();

                // Leggo tutti i byte e riempio l'array
                byte[] data = new byte[l];
                int r = 0;
                int o = 0;
                while ((r = stream.Read(data, o, data.Length - o)) > 0)
                {
                    o += r;
                    if (o >= data.Length) break;
                }

                Debug.Print("Frame");
                // Scateno l'evento
                CameraFrame(new Bitmap(data, Bitmap.BitmapImageType.Jpeg));
            }
        }

    }
}
